"""
WebFrame application options

Below is a list of options that WebFrame accepts and their default values.

"""
opt_space = {
    "http.default_encoding" : "utf-8",
    "http.query_handler_prefix" : "ACTION:",
    "file.default_encoding" : "utf-8",
    "template.allow_pathinfo" : True,
    "template.export_ns" : False,
    "logging.level" : None,
    "logging.format" : "%(message)s",
    "json.content_type" : "application/x-json",
    "xml.pretty" : True, 
    "xml.autostrip" : True,
    "xml.compact" : False,
    "auth.get_auth_user_sql" : ("SELECT * FROM User " 
                                +" WHERE user=%(user)s "
                                +" AND passwd=password(%(passwd)s) "),
    "auth.remove_user_colums" : ["passwd"],
    "auth.form_login_helpers" : True,
    "auth.default_login_method" : "loginredir",
    "auth.loginpage_path" : "/login.html",
    "auth.loginredir_uri" : "/login.html",
    "session.sql_get" : "SELECT data FROM Session WHERE sid=%s",
    "session.sql_set" : "REPLACE INTO Session SET data=%s, sid=%s",
    "session.timeout" : 28800,  # 8h in seconds
    "session.randomchars" : "CHANGE ME!!!!", # random string
                                 # used for session id generation
                                 # put whatever you want here
    "debug" : True
}

__docs = {
    "http.query_handler_prefix" :
    """
    What prefix HTML form / querystring parameters must have to
    trigger the search for query string handlers.
    """,
    
    "file.default_encoding" :
    """
    Default encoding for HTML template files
    """,
    
    "template.allow_pathinfo" :
    """
    Determines wether or not path-info information is allowed for html
    templates. Setting to true allows '/tmpl.html/extra/paths' and
    puts the path information into the templates namespace. 
    """,
    
    "template.export_ns" :
    """
    If set to true the dict containing the template namespace is
    exported to the template as '__ns'. This might be useful for
    debugging but is not a good idea to have enabled in production.
    """,
    
    "logging.level" :
    """
    A debug level from the 'logging' python module. If set to None the
    system defaults are used and are dependant on if the 'debug' opt
    is set.
    """,
    
    "logging.format" :
    """
    The format of the log messages. WebFrame uses the python logging
    module, so it accepts the same syntax for the format.
    """,
    
    "json.content_type" :
    """
    Default content type set on the response from JSON type handlers
    """,
    
    "xml.pretty" :
    """
    Format xml from XML type handlersto look pretty before sending it
    to the client. Good for debugging, but may cause problems with
    some badly broken clients.
    """,
    
    "xml.autostrip" :
    """
    If true the text value/contents of a xml tag is stripped by the
    system before being handed over to the XML handler. I.e
    <tag>  something  </tag> is converted to <tag>something</tag>.
    If False the handler must do stripping itself if it needs it
    """,

    "xml.compact" :
    """
    Compact the XML sent to the client. This is basically the opposite
    of the 'xml.pretty' option.
    """,
    
    "auth.get_auth_user_sql" :
    """
    This defines the SQL query to be used by the user/auth helper
    functions. The SQL query should accept a username and password and
    return a row only if the user is found and the password is
    correct. See WebFrame.auth for more info.
    """,
    
    "auth.remove_user_colums" :
    """
    A list of column names to remove from the resultset before
    putting it into the User object. Passwords and other sensitive
    information should be specified here so it is not leaked.
    Another, more secure way is to change the SQL query to only fetch
    the columns that are of interrest to the user object.
    """,
    
    "auth.form_login_helpers" :
    """
    If true it enables helper query-string handlers for the LOGIN and
    LOGOUT action. See WebFrame.auth for more info.
    """,
    
    "auth.default_login_method" :
    """
    The default login method to use if no other login method has been
    specified by the access policy that triggers the login request.
    The possible values are the same as those available in the access
    policy specification. See WebFrame.auth for more info.
    """,

    "auth.loginpage_path" :
    """
    The template file containing the login page to be sent to the
    client when the login method is set to 'loginpage'
    """,
    
    "auth.loginredir_uri" :
    """
    The uri to redirect the client to so they can log in
    """,
    
    "session.sql_get" :
    """
    A SQL query that returns a column called 'data' with the session
    data associated with a particular session ID.
    """,
    
    "session.sql_set" :
    """
    The SQL query used to update the session data for a given session.
    """,
    
    "session.timeout" :
    """
    The timeout in seconds for the session cookie.
    NOTE: WebFrame doesn't remove session records from the
    database. You will have to do so yourself with a cron-job or
    something simmilar.
    """,
    
    "session.randomchars" :
    """
    A string of random characters used to provide additional
    'unguessabillity' to the session ID generator.
    This should be changed to something random for each installation
    of a WebFrame app to minimize risk of people guessing session ids
    """,
    
    "debug" :
    """
    If true, enables debug output
    """
    }

# Setup documentation
__list = opt_space.keys()
__list.sort()
for __i in __list:
    __doc__ += "%s = %s\n" % (__i, str(opt_space[__i]))
    if __i in __docs:
        __text = __docs[__i].strip()
        __text = __text.splitlines()
        __text = [__tl.strip() for __tl in __text]
        __text = "\n\t".join(__text)
        __doc__ += "\t"+__text+"\n"
    __doc__ += "\n"
