import urllib
import WebFrame as WF
import logging
import json
try:
    from xml.etree import cElementTree as ET
except ImportError:
    try:
        from elementtree import cElementTree as ET
    except ImportError:
        from elementtree import ElementTree as ET

class StopProcessing(Exception):
    def __init__(self, msg=None):
        self.message = msg

class Request(object):
    """The HTTP request object.

    Keeps track of all resources related to this request and allows you
    to access them. They are all read-only.
    """
    uri = 'The uri for this request'
    method = 'HTTP request method'
    content_length = 'Content length, in bytes, of request data'
    filename = 'The translated filename for the page accessed'
    docroot = 'Webspace document root'
    cookies = 'The cookie jar'

    def __init__(self):
        """Do not create this object yourself, thats the systems job"""
        self.params = {}
        self._body = None

    def _parse_query(self):
        if self.query_string:
            # We need to parse URL ourselves
            query = self.query_string.replace("?", "&") 
            for qp in query.split("&"):
                try:
                    key, val = qp.split("=")
                    val = urllib.unquote_plus(val)
                except ValueError:
                    pass # Ignore broken shit
                else:
                    # Insert into params
                    if key in self.params:
                        self.params[key].append(val)
                    else:
                        self.params[key] = [val]

    def body(self):
        """Return the request body.
        None is returned if the request had no body"""
        return self._body

    def body_xml(self):
        """Parse the body as XML and return an elementtree root element"""
        if self._body == None:
            return None
        xml = ET.XML(self._body)
        if WF.get_opt("xml.autostrip"):
            self._strip_xml(xml)
        return xml

    def body_json(self):
        """Parse the body as JSON, and return a deserialized structure"""
        if self._body == None:
            return None
        js = json.read(self._body)
        return js

    def get_cookie(self, cookie):
        """Get a named cookie from the cookie jar"""
        return self.cookies[cookie]

    def _strip_xml(self, xml):
        if len(xml) == 0:
            xml.text = xml.text.strip()
        else:
            for e in xml:
                self._strip_xml(e)

class Response:
    HTTP_PERMANENT_REDIRECT = 301
    HTTP_TEMPORARY_REDIRECT = 302
    HTTP_UNAUTHORIZED = 401
    HTTP_FORBIDDEN = 403
    HTTP_NOT_FOUND = 404

    def __init__(self, request):
        """Do not create this object. The system  does that"""
        self.processing = True
        self.request = request
        self.content_type = "text/html"
        self.encoding = WF.get_opt("http.default_encoding")

    def _set_ct(self):
        ct = self.content_type
        ct += '; charset="'+self.encoding+'"'
        self._do_set_ct(ct)

    def _do_set_ct(self, ct):
        """This must be implemented by the webserver API"""
        raise NotImplemented

    def _compact_xml(self, elem):
        if elem.text:
            elem.text = elem.text.strip()
        if elem.tail:
            elem.tail = elem.tail.strip()
        for elem in elem:
            self._compact_xml(elem)

    def _pretty_xml(self, elem, level=0):
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self._pretty_xml(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i
                    

    def write(self, data):
        """Write a text string to the client.
        The string is encoded using the encoding set by the application"""
        self._set_ct()
        if not isinstance(data, unicode) and not isinstance(data, str):
            data = unicode(data)
        self._do_write(data.encode(self.encoding, "replace"))

    def _do_write(self, data):
        """This must be implemented by the webserver API"""
        raise NotImplemented
        
    def status(self, status):
        """Set the HTTP status code for this response
        Should rarely be used directly
        This must be implemented by the webserver API"""
        raise NotImplemented

    def set_header(self, header, value):
        """Set a HTTP request header."""
        raise NotImplemented

    def write_binary(self, data):
        """Write a binary blob of data. No conversion is performed"""
        self._set_ct()
        self._do_write(data)

    def write_xml(self, xml):
        """Serializes a elementtree root element into an XML document
        and sends that to the client"""
        self._set_ct()
        if WF.get_opt("xml.pretty"):
            self._pretty_xml(xml)
        if WF.get_opt("xml.compact"):
            self._compact_xml(xml)
        self.write_binary(ET.tostring(xml, encoding=self.encoding))

    def write_json(self, js):
        """Serializes a structure as JSON and sends that to the client"""
        self.write(json.write(js))

    def set_cookie(self, name, value, timestamp):
        """Sets a cookie.
        You must call this before calling any of the write methods
        since those aren't currently buffered
        This must be implemented by webserver API"""
        raise NotImplemented

    def stop_processing(self):
        """Use this to tell the system that no more processing should
        be done for this request"""
        self.processing = False
        raise StopProcessing

    def not_found(self, text="NOT FOUND"):
        """Return a HTTP 404 response, with an optional body"""
        self.processing = False
        self.status(self.HTTP_NOT_FOUND)
        self.write(text)
        raise StopProcessing

    def access_denied(self, text="ACCESS DENIED"):
        """Return a HTTP FORBIDDEN response, with an optional body"""
        self.processing = False
        self.status(self.HTTP_FORBIDDEN)
        self.write(text)
        raise StopProcessing

    def login_required(self, text="LOGIN REQUIRED"):
        """Return a HTTP UNAUTHORIZED response, asking the user to log in"""
        self.processing = False
        self.status(self.HTTP_UNAUTHORIZED)
        self.write(text)
        raise StopProcessing

    def cont(self):
        """Tell the system that processing should continue for this request"""
        self.processing = True
        return True

    def redir(self, uri, code=HTTP_TEMPORARY_REDIRECT):
        """Redirect to a specified URI"""
        self.set_header("Location", uri)
        self.status(code)
        self.processing = False
        raise StopProcessing
